def get_docsplitter_pipe(
    chunk_size: int = 500,
    chunk_overlap: int = 0,
    split_patterns: list = ["\n\n", "\n", " ", ""],
    explode_splits: bool = True,
    keep_separators: bool = True,
    patterns_are_regex: bool = False,
    trim_whitespace: bool = True,
):
    from johnsnowlabs import nlp

    nlp.start()

    return nlp.LightPipeline(
        nlp.PipelineModel(
            stages=[
                nlp.DocumentAssembler().setInputCol("text").setOutputCol("document"),
                (nlp.DocumentCharacterTextSplitter()
                 .setInputCols(["document"])
                 .setOutputCol("splits")
                 .setChunkSize(chunk_size)
                 .setChunkOverlap(chunk_overlap)
                 .setExplodeSplits(explode_splits)
                 .setPatternsAreRegex(patterns_are_regex)
                 .setSplitPatterns(split_patterns)
                 .setTrimWhitespace(trim_whitespace)
                 .setKeepSeparators(keep_separators)
                 )
            ]
        )
    )


def get_doc_splitter_internal_pipe(
    split_mode: str = "recursive",
    chunk_size: int = 500,
    chunk_overlap: int = 0,
    split_patterns: list = None,    # different value for split modes
    explode_splits: bool = True,
    keep_separators: bool = True,
    patterns_are_regex: bool = False,
    trim_whitespace: bool = True,
    sentence_awareness: bool = False,
    max_length: int = None,         # different value for split modes
    custom_bounds_strategy: str = "prepend",
    case_sensitive: bool = False,
    meta_data_fields: list = [],
    enable_sentence_increment: bool = False,
    # Tokenizer Params
    tokenizer_target_pattern: str = None,
    tokenizer_prefix_pattern: str = None,
    tokenizer_suffix_pattern: str = None,
    tokenizer_infix_patterns: list = None,
    tokenizer_exceptions: list = None,
    tokenizer_exceptions_path: str = None,
    tokenizer_case_sensitive_exceptions: bool = None,
    tokenizer_context_chars: list = None,
    tokenizer_split_pattern: str = None,
    tokenizer_split_chars: list = None,
    tokenizer_min_length: int = None,
    tokenizer_max_length: int = None,
    # Sentence Detector Params
    sentence_model_architecture: str = None,
    sentence_explode_sentences: bool = None,
    sentence_custom_bounds: list = None,
    sentence_use_custom_bounds_only: bool = None,
    sentence_split_length: int = None,
    sentence_min_length: int = None,
    sentence_max_length: int = None,
    sentence_impossible_penultimates: list = None,
):
    from johnsnowlabs import nlp, medical

    spark = nlp.start()
    split_modes = ["char", "sentence", "token", "recursive", "regex"]
    stages = [nlp.DocumentAssembler().setInputCol("text")]
    if split_mode not in split_modes:
        raise ValueError("split_mode must be one of %r." % split_modes)
    splitter_input_cols = ["document"]

    if split_mode == "sentence" or (sentence_awareness and split_mode in ["char", "sentence", "token"]):
        sent_detector = (
            nlp.SentenceDetectorDLModel.pretrained(
                "sentence_detector_dl_healthcare", "en", "clinical/models"
            )
            .setInputCols(["document"])
            .setOutputCol("sentence")
        )
        splitter_input_cols.append("sentence")

        if sentence_model_architecture:
            sent_detector.setModelArchitecture(sentence_model_architecture)
        if sentence_explode_sentences:
            sent_detector.setExplodeSentences(sentence_explode_sentences)
        if sentence_custom_bounds:
            sent_detector.setCustomBounds(sentence_custom_bounds)
        if sentence_use_custom_bounds_only:
            sent_detector.setUseCustomBoundsOnly(sentence_use_custom_bounds_only)
        if sentence_split_length:
            sent_detector.setSplitLength(sentence_split_length)
        if sentence_min_length:
            sent_detector.setMinLength(sentence_min_length)
        if sentence_max_length:
            sent_detector.setMaxLength(sentence_max_length)
        if sentence_impossible_penultimates:
            sent_detector.setImpossiblePenultimates(sentence_impossible_penultimates)
        # Add SentenceDetector to pipeline
        stages.append(sent_detector)

    if split_mode == "token":
        tokenizer = nlp.Tokenizer().setInputCols([splitter_input_cols[-1]]).setOutputCol("token")
        splitter_input_cols.append("token")
        if tokenizer_target_pattern:
            tokenizer.setTargetPattern(tokenizer_target_pattern)
        if tokenizer_prefix_pattern:
            tokenizer.setPrefixPattern(tokenizer_prefix_pattern)
        if tokenizer_suffix_pattern:
            tokenizer.setSuffixPattern(tokenizer_suffix_pattern)
        if tokenizer_infix_patterns:
            tokenizer.setInfixPatterns(tokenizer_infix_patterns)
        if tokenizer_exceptions:
            tokenizer.setExceptions(tokenizer_exceptions)
        if tokenizer_exceptions_path:
            tokenizer.setExceptionsPath(tokenizer_exceptions_path)
        if tokenizer_case_sensitive_exceptions:
            tokenizer.setCaseSensitiveExceptions(tokenizer_case_sensitive_exceptions)
        if tokenizer_context_chars:
            tokenizer.setContextChars(tokenizer_context_chars)
        if tokenizer_split_pattern:
            tokenizer.setSplitPattern(tokenizer_split_pattern)
        if tokenizer_split_chars:
            tokenizer.setSplitChars(tokenizer_split_chars)
        if tokenizer_min_length:
            tokenizer.setMinLength(tokenizer_min_length)
        if tokenizer_max_length:
            tokenizer.setMaxLength(tokenizer_max_length)
        # Add tokenizer to pipeline
        stages.append(tokenizer)

    doc_splitter = (
        medical.DocumentSplitter()
        .setInputCols(splitter_input_cols)
        .setOutputCol("splits")
        .setSplitMode(split_mode)
        .setChunkSize(chunk_size)
        .setChunkOverlap(chunk_overlap)
        .setExplodeSplits(explode_splits)
        .setPatternsAreRegex(patterns_are_regex)
        .setTrimWhitespace(trim_whitespace)
        .setKeepSeparators(keep_separators)
        .setSentenceAwareness(sentence_awareness)
        .setCustomBoundsStrategy(custom_bounds_strategy)
        .setCaseSensitive(case_sensitive)
        .setMetaDataFields(meta_data_fields)
        .setEnableSentenceIncrement(enable_sentence_increment)
    )
    if split_patterns:
        doc_splitter.setSplitPatterns(split_patterns)
    if max_length:
        doc_splitter.setMaxLength(max_length)
    # Add DocumentSplitter to pipeline
    stages.append(doc_splitter)

    return nlp.LightPipeline(
        # nlp.PipelineModel(stages=stages)
        nlp.Pipeline(stages=stages).fit(spark.createDataFrame([[""]]).toDF("text"))
    )



import importlib
from abc import ABC
from typing import Dict, Union

from johnsnowlabs.py_models.lib_version import LibVersion
from johnsnowlabs.py_models.url_dependency import UrlDependency
from johnsnowlabs.utils.enums import *


def is_spark_version_env(spark_version: str) -> bool:
    import pyspark

    env_spark_version = pyspark.__version__[0:-2].replace(".", "")
    return spark_version in env_spark_version


def try_import_lib(lib: str, print_failure=False):
    try:
        importlib.import_module(lib)
        return True
    except Exception as _:
        if print_failure:
            print(f"Failed to import {lib}. Seems not installed.")


class Py4JJslLibDependencyResolverABC(ABC):
    """
    We define a resolver for all Py4J based dependencies
    Each JslLibDependencyResolver util must implement the following :
    - get_jar_urls(lib_version,spark_version_to_match,Optional[secret])->UrlDependency
    - get_mvn_coordinates(lib_version,spark_version_to_match,Optional[secret])->RepoDependency
    - get_python_urls(lib_version,spark_version_to_match,Optional[secret],install_type)->UrlDependency
    - get_pypi_identifier(lib_version,spark_version_to_match,Optional[secret],install_type)->RepoDependency
    - get_lib_version_i

    INVARIANTS :
    2. Jar/Py Dependencies are always Spark version UN-agnostic.
     This is why we have to map pyspark version to jar/py dependencies


    """

    has_gpu_jars: bool = False
    has_cpu_jars: bool = False
    has_m1_jar: bool = False
    has_py_install: bool = False
    has_secret: bool = False
    lib_version: LibVersion
    product_name: ProductName
    # key = Supported Spark Version, Value = Dict  with key=m1/cpu/gpu and value = URL/ URL formatable with secret
    compatible_spark_to_jar_map: Dict[
        SparkVersion, Dict[JvmHardwareTarget, UrlDependency]
    ]

    # key = Supported Spark Version, Value = Dict with key=tar/wheel and value = URL? URL formatable with secret
    compatible_spark_to_py_map: Dict[SparkVersion, Dict[PyInstallTypes, UrlDependency]]

    @classmethod
    def get_dependency_url(
        cls,
        install_type: Union[JvmHardwareTarget, PyInstallTypes],
        spark_version_to_match: Optional[Union[str, LibVersion]],
        secret: Optional[str] = None,
    ) -> UrlDependency:

        if install_type in PyInstallTypes:
            return cls.get_py_urls(
                secret=secret,
                spark_version_to_match=spark_version_to_match,
                install_type=install_type,
            )
        elif install_type in JvmHardwareTarget:
            return cls.get_jar_urls(
                secret=secret,
                spark_version_to_match=spark_version_to_match,
                hardware_target=install_type,
            )
        else:
            raise ValueError(
                f"Invalid Install type = {install_type} must be JvmHardwareTarget or PyInstallType"
            )

    @classmethod
    def get_url_from_compat_map(
        cls,
        compat_map: Dict[
            SparkVersion, Dict[Union[JvmHardwareTarget, PyInstallTypes], UrlDependency]
        ],
        install_type: Union[JvmHardwareTarget, PyInstallTypes],
        spark_version_to_match: Optional[Union[str, LibVersion]],
        secret: Optional[str] = None,
        suffix: str = "",
    ) -> UrlDependency:

        spark_version_to_match = cls.resolve_to_spark_lib_version(
            spark_version_to_match
        )
        matching_jsl_spark_release = None
        for compatible_spark in compat_map.keys():
            compatible_spark: SparkVersion = compatible_spark
            if compatible_spark.value.equals(spark_version_to_match):
                matching_jsl_spark_release = compatible_spark

        if not matching_jsl_spark_release:
            # Todo make special type of exception and catch?
            # TODO nicer print
            raise Exception(
                f"{cls.product_name.value} does not have any install candidates for"
                f" pyspark=={spark_version_to_match.as_str()} \n"
                f"Please install one of the following Pyspark Versions : {1} TODO"
            )

        if cls.has_secret:
            if settings.enforce_versions:
                url = compat_map[matching_jsl_spark_release][install_type].url.format(
                    secret=secret, lib_version=cls.lib_version.as_str()
                )
            else:
                # Parse Version from secret
                url = compat_map[matching_jsl_spark_release][install_type].url.format(
                    secret=secret, lib_version=secret.split("-")[0].replace(".PR",'')
                )

        else:
            if settings.enforce_versions:
                url = compat_map[matching_jsl_spark_release][install_type].url.format(
                    lib_version=cls.lib_version.as_str()
                )
            else:
                # read from updated settings instead of already instantiated Objects, which will not reflect setting updates
                url = compat_map[matching_jsl_spark_release][install_type].url.format(
                    lib_version=LatestCompatibleProductVersion.from_settings(
                        cls.product_name
                    )
                )
                url.replace(
                    cls.lib_version.as_str(),
                    LatestCompatibleProductVersion.from_settings(cls.product_name),
                )

        # NAME-VERSION-INSTALL_TYPE-for-spark-SPARK_VERSION-.[jar/tar/wheel]
        name = (
            f"{cls.product_name.value}-{cls.lib_version.as_str()}"
            f"-{install_type.name}-for-spark-{matching_jsl_spark_release.value.as_str()}.{suffix}"
        )
        return UrlDependency(
            url=url,
            dependency_type=install_type,
            spark_version=matching_jsl_spark_release,
            dependency_version=cls.lib_version,
            file_name=name,
            product_name=cls.product_name.value,
        )

    @classmethod
    def get_jar_urls(
        cls,
        secret: Optional[str] = None,
        spark_version_to_match: Optional[Union[str, LibVersion]] = None,
        hardware_target: JvmHardwareTarget = JvmHardwareTarget.cpu,
    ) -> UrlDependency:
        """
        Get jar URL location for hardware target.
        Jar loc formats with secret+jsl_lib_version for licensed and
         only with jsl-lib_version for open source
        """
        if hardware_target == JvmHardwareTarget.cpu and not cls.has_cpu_jars:
            raise Exception(f"{cls.product_name.value} has no CPU Jars!")

        if hardware_target == JvmHardwareTarget.gpu and not cls.has_gpu_jars:
            raise Exception(f"{cls.product_name.value} has no GPU Jars!")

        if hardware_target == JvmHardwareTarget.m1 and not cls.has_m1_jar:
            raise Exception(f"{cls.product_name.value} has no M1 Jars!")

        return cls.get_url_from_compat_map(
            compat_map=cls.compatible_spark_to_jar_map,
            install_type=hardware_target,
            spark_version_to_match=spark_version_to_match,
            secret=secret,
            suffix="jar",
        )

    @classmethod
    def get_py_urls(
        cls,
        secret: Optional[str] = None,
        spark_version_to_match: Optional[Union[str, LibVersion]] = None,
        install_type: PyInstallTypes = PyInstallTypes.wheel,
    ) -> UrlDependency:
        if not cls.has_py_install:
            raise Exception(f"{cls.product_name.value} has no Py Dependencies!")
        return cls.get_url_from_compat_map(
            compat_map=cls.compatible_spark_to_py_map,
            install_type=install_type,
            spark_version_to_match=spark_version_to_match,
            secret=secret,
            suffix=install_type.value,
        )

    @classmethod
    def resolve_to_spark_lib_version(cls, spark_version_to_match) -> LibVersion:
        """Get Pyspark version from installed pyspark.
         If not compatible across all libraries, print warning.
         If no pyspark installed, use latest pyspark which is compatible across all libs.
        Uses LatestCompatibleProductVersion for this

        :return: LibVersionIdentifier for the specific Resolver
        """

        # 1. Check if is spark_version_to_match is a string, if yes cast to LibVersion
        if isinstance(spark_version_to_match, str):
            # TODO print warning if spark_version_to_match is not compatible across all libs
            #  in --> Helper func since we need it in get_installed_pyspark_version_or_latest_compatible as well
            return LibVersion(spark_version_to_match)
        # 2. Check if is spark_version_to_match is a LibVersion, if yes return
        elif isinstance(spark_version_to_match, LibVersion):
            return spark_version_to_match
        # 3. Check if is SparkVersion enum
        elif isinstance(spark_version_to_match, SparkVersion):
            return spark_version_to_match.value
        # 3. Check if is spark_version_to_match is some other type, if yes raise exception, should not happen
        elif spark_version_to_match:
            raise ValueError(
                f"Invalid Type for spark_version_to_match, "
                f"must be either str in format A.B.C, None or Libversion"
                f"But type is ={type(spark_version_to_match)}"
            )

        # 4. Check if is pyspark is installed
        elif try_import_lib("pyspark"):
            import pyspark

            # TODO check product wide compatibility for pre-installed pyspark --> make helper method for that
            return LibVersion(pyspark.__version__)
        # 5. Return the latest compatible pyspark if no other method  resolves
        else:
            return LatestCompatibleProductVersion.pyspark.value



import subprocess
import sys
from pathlib import Path
from typing import List, Callable

import colorama
import pandas as pd

import johnsnowlabs.utils.testing.test_settings
from johnsnowlabs.utils.file_utils import str_to_file

Path(johnsnowlabs.utils.testing.test_settings.tmp_markdown_dir).mkdir(
    exist_ok=True, parents=True
)


def run_cmd_and_check_succ(
    args: List[str],
    log=True,
    suc_print=johnsnowlabs.utils.testing.test_settings.success_worker_print,
    return_pipes=False,
    shell=False,
    raise_on_fail=False,
    use_code=False,
) -> bool:
    if log:
        if len(args) == 1:
            print(
                f"👷 Executing {colorama.Fore.LIGHTGREEN_EX}{args[0]}{colorama.Fore.RESET}"
            )
        else:
            print(
                f"👷 Executing {colorama.Fore.LIGHTGREEN_EX}{args}{colorama.Fore.RESET}"
            )

    r = subprocess.run(args, capture_output=True, shell=shell)
    was_suc = process_was_suc(r, suc_print=suc_print, use_code=use_code)
    if log:
        if was_suc:
            print(
                f"{colorama.Fore.LIGHTGREEN_EX}✅ Success running {args}{colorama.Fore.RESET}"
            )
        else:
            print(
                f"{colorama.Fore.LIGHTRED_EX}❌ Failure running {args}{colorama.Fore.LIGHTGREEN_EX}"
            )

    if log:
        log_process(r)
    if raise_on_fail and not was_suc:
        raise ValueError(f"Failed running {args}")
    if return_pipes:
        return was_suc, r
    return was_suc


def process_was_suc(
    result: subprocess.CompletedProcess,
    suc_print=johnsnowlabs.utils.testing.test_settings.success_worker_print,
    use_code=False,
) -> bool:
    if use_code and result.returncode != 0:
        return False
    elif use_code and result.returncode == 0:
        return True
    else:
        return suc_print in result.stdout.decode()


def log_process(result: subprocess.CompletedProcess):
    print("______________STDOUT:")
    print(result.stdout.decode())
    print("______________STDERR:")
    print(result.stderr.decode())


# def execute_slave_test(py_cmd):
#     prefix = 'from johnsnowlabs import * \n'
#     postfix = f"\neval_class('{py_cmd}') \n"
#     script_file_name = 'test_script.py'
#     script = inspect.getsource(eval_class)
#     script = f'{prefix}{script}{postfix}'
#     print(script)
#     str_to_file(script, script_file_name)
#     return run_cmd_and_check_succ(['python', script_file_name])
#


def execute_function_as_new_proc(
    function: Callable,
    suc_print=johnsnowlabs.utils.testing.test_settings.success_worker_print,
):
    pass


def execute_py_script_string_as_new_proc(
    py_script,
    suc_print=johnsnowlabs.utils.testing.test_settings.success_worker_print,
    py_exec_path=sys.executable,
    log=True,
    file_name=None,  # Optional metadata
    use_i_py=False,
    add_prefix=True,
):
    if file_name:
        out_path = f"{johnsnowlabs.utils.testing.test_settings.tmp_markdown_dir}/{file_name}_MD_TEST.py"
    else:
        out_path = "tmp.py"
    if add_prefix:
        prefix = """
from johnsnowlabs import *
spark = nlp.start()
"""
    else:
        prefix = ""

    suffix = f"""
print('{suc_print}')    
    
"""

    str_to_file(prefix + py_script + suffix, out_path)
    suc, proc = execute_py_script_as_new_proc(
        out_path, use_i_py=use_i_py, py_exec_path=py_exec_path
    )
    return make_modelhub_snippet_log(file_name, suc, proc)


def execute_py_script_as_new_proc(
    py_script_path: str,
    suc_print=johnsnowlabs.utils.testing.test_settings.success_worker_print,
    py_exec_path=sys.executable,
    log=True,
    use_i_py=True,
):
    # requires ipython installed
    if use_i_py:
        cmd_args = [py_exec_path, "-m", "IPython", py_script_path]
    else:
        cmd_args = [py_exec_path, py_script_path]  # '-m', 'IPython',
    return run_cmd_and_check_succ(
        cmd_args, log=log, suc_print=suc_print, return_pipes=True
    )


def log_multi_run_status(run_df):
    num_fails = len(run_df[run_df.success == False])
    print(f"#" * 10 + f"RUN RESULTS {num_fails} Failures!" + "#" * 10)
    i = 0
    for idx, row in run_df[run_df.success == False].iterrows():
        print(f'{"!" * 10} Failure No {i} : {"!" * 10}')
        i += 1
        for col in run_df.columns:
            print(f"{col} : {row[col]}")


def make_modelhub_snippet_log(md_file, suc, proc):
    return {
        "md_file": md_file,
        "success": suc,
        "stdout": proc.stdout.decode(),
        "stderr": proc.stderr.decode(),
    }


# def test_list_of_py_script_path(py_sc)
def test_list_of_py_script_strings(py_script_paths, use_i_py=False):
    total = len(py_script_paths)
    df = []
    for i, p in enumerate(py_script_paths):
        print(f"Testing {i}/{total}")
        df.append(
            execute_py_script_string_as_new_proc(p, file_name=f"{i}_TEST.py"),
            use_i_py=use_i_py,
        )
    return pd.DataFrame(df)



import unittest

from haystack import Document
from mlflow.models import ModelSignature, infer_signature
import os
from haystack.pipelines import RayPipeline

# https://docs.haystack.deepset.ai/docs/pipelines#distributed-pipelines-with-ray
from haystack.nodes import EmbeddingRetriever

from johnsnowlabs import nlp
from johnsnowlabs.frameworks.embedding_retrieval.haystack_node import (
    JohnSnowLabsHaystackDocumentSplitter, JohnSnowLabsHaystackMedicalDocumentSplitter,
)

# os.environ["PROTOCOL_BUFFERS_PYTHON_IMPLEMENTATION"] = "python"
from johnsnowlabs.llm import embedding_retrieval

class HaystackTestCase(unittest.TestCase):
    def get_docs(self):
        return [
            Document(
                content="I like apples",
                content_type="text",
                id=1,
            ),
            Document(
                content="I like bananas \n\n and other things \n\n like icream \n\n and cats",
                content_type="text",
                id=2,
            ),
        ]


    def test_integration(self):
        from haystack.nodes import PreProcessor
        from haystack import Pipeline
        from haystack.document_stores import InMemoryDocumentStore

        # processor = PreProcessor(
        #     clean_whitespace=True,
        #     split_by="word",
        # )

        processor = JohnSnowLabsHaystackDocumentSplitter()

        # Write some processed data to Doc store, so we can retrieve it later
        document_store = InMemoryDocumentStore(embedding_dim=768)
        document_store.write_documents(processor.process(self.get_docs()))

        # could just use EmbeddingRetriever but mehhh
        retriever = embedding_retrieval.JohnSnowLabsHaystackEmbedder(
            embedding_model="en.embed_sentence.bert_base_uncased",
            # model_format="johnsnowlabs",
            document_store=document_store,
            use_gpu=False,
        )
        document_store.update_embeddings(retriever)

        pipe = Pipeline()
        # pipe.add_node(component=processor, name="Preprocess", inputs=["Query"])
        pipe.add_node(component=processor, name="Preprocess", inputs=["Query"])
        pipe.add_node(component=retriever, name="Embed&Retrieve", inputs=["Query"])
        result = pipe.run(documents=self.get_docs(), query="lol")
        print(result)

    def test_doc_splitter(self):
        splitter = JohnSnowLabsHaystackDocumentSplitter(chunk_size=20)
        docs = splitter.process(self.get_docs())
        print(docs)

    def test_medical_doc_splitter(self):
        splitter = JohnSnowLabsHaystackMedicalDocumentSplitter(split_mode="regex")
        docs = splitter.process(self.get_docs())
        print(docs)



import os
import unittest
from unittest.mock import patch

from johnsnowlabs import nlp, settings
from johnsnowlabs.py_models.jsl_secrets import JslSecrets


class EnforceVersions(unittest.TestCase):
    def setUp(self) -> None:
        nlp.settings.enforce_versions=True

    @patch("johnsnowlabs.py_models.jsl_secrets.JslSecrets.enforce_versions")
    def test_should_try_to_enforce_versions(self, mock_enforce_versions):
        nlp.install(med_license=os.environ.get("TEST_VALID_LICENSE"), only_refresh_credentials=True, refresh_install=True)
        self.assertTrue(mock_enforce_versions.called)

    def test_should_install_compatible_secrets_with_valid_license(self):
        nlp.install(med_license=os.environ.get("TEST_VALID_LICENSE"), only_refresh_credentials=True, refresh_install=True)
        secrets = JslSecrets.from_jsl_home()
        self.assertEqual(secrets.HC_LICENSE, os.environ.get("TEST_VALID_LICENSE"))
        self.assertEqual(secrets.HC_VERSION, settings.raw_version_secret_medical)
        self.assertEqual(secrets.OCR_VERSION, settings.raw_version_secret_ocr)
    
    def test_should_fallback_to_existing_secrets_with_invalid_license(self):
        nlp.install(med_license=os.environ.get("TEST_INVALID_LICENSE"), 
                    ocr_secret="random-ocr-secret",
                    enterprise_nlp_secret="random-medical-secret",
                    only_refresh_credentials=True, refresh_install=True)
        secrets = JslSecrets.from_jsl_home()
        self.assertTrue(secrets.HC_LICENSE, os.environ.get("TEST_INVALID_LICENSE"))
        self.assertEqual(secrets.HC_SECRET, "random-medical-secret")
        self.assertEqual(secrets.OCR_SECRET, "random-ocr-secret")
    

class DonotEnforceVersions(unittest.TestCase):
    def setUp(self) -> None:
        nlp.settings.enforce_versions=False
        self.enterprise_nlp_secret = "nlp_random"
        self.ocr_secret = "ocr_random"

    def test_should_install_incompatible_secrets_from_input(self):
        nlp.install(med_license=os.environ.get("TEST_VALID_LICENSE"),ocr_secret=self.ocr_secret, enterprise_nlp_secret=self.enterprise_nlp_secret, visual=True , only_refresh_credentials=True, refresh_install=True)
        secrets = JslSecrets.from_jsl_home()
        self.assertEqual(secrets.HC_LICENSE, os.environ.get("TEST_VALID_LICENSE"))
        self.assertEqual(secrets.HC_SECRET, self.enterprise_nlp_secret)
        self.assertEqual(secrets.OCR_SECRET, self.ocr_secret)
    
    @patch("johnsnowlabs.py_models.jsl_secrets.JslSecrets.enforce_versions")
    @patch("johnsnowlabs.py_models.jsl_secrets.JslSecrets.is_ocr_secret_correct_version")
    @patch("johnsnowlabs.py_models.jsl_secrets.JslSecrets.is_hc_secret_correct_version")
    def test_should_not_try_to_enforce_versions_if_input_secrets_are_compatible(self, mock_hc_secret_correct, mock_ocr_secret_correct, mock_enforce_versions):
        nlp.install(med_license=os.environ.get("TEST_VALID_LICENSE"), only_refresh_credentials=True, refresh_install=True)
        mock_hc_secret_correct.return_value = True
        mock_ocr_secret_correct.return_value = True
        self.assertFalse(mock_enforce_versions.called)



from johnsnowlabs import *
import unittest

from johnsnowlabs import settings

from johnsnowlabs.py_models.install_info import  InstallFolder


class MyTestCase(unittest.TestCase):
    def test_something(self):
        p = '/home/ckl/.johnsnowlabs/java_installs/info2.json'
        p = '/home/ckl/.johnsnowlabs/java_installs/info3.json'
        info = FolderInfoV2.parse_file(p)
        print(info)
        print('_'*20)
        j = info.json(indent=4)
        print(j)

        p = '/home/ckl/.johnsnowlabs/java_installs/info9.json'
        info.write(p,indent=4)

    def test_parse_install_folder(self):
        f = '/home/ckl/old_home/ckl/Documents/freelance/johnsnowlabs_lib/tests/utils/spark_nlp_ocr_hc.json'
        # jsl.install()
        java_info = InstallFolder.parse_file(settings.java_info_file)
        print(java_info)
        py_info = InstallFolder.parse_file(settings.py_info_file)

if __name__ == '__main__':
    unittest.main()



import os

from johnsnowlabs import settings, nlp

settings.enforce_versions = False

nlp_license = os.environ.get("JOHNSNOWLABS_LICENSE", None)
nlp_secret = os.environ.get("MEDICAL_SECRET", None)
visual_secret = os.environ.get("VISUAL_SECRET", None)
aws_access_key_id = os.environ.get("JOHNSNOWLABS_AWS_ACCESS_KEY_ID", None)
aws_secret_access_key = os.environ.get("JOHNSNOWLABS_AWS_SECRET_ACCESS_KEY", None)
HARDWARE_TARGET = os.environ.get("HARDWARE_TARGET", "cpu")
model_ref = os.environ.get("MODEL_TO_LOAD", None)

nlp.install(
    browser_login=False,
    force_browser=False,
    med_license=nlp_license,
    enterprise_nlp_secret=nlp_secret,
    ocr_secret=visual_secret,
    visual=True if visual_secret else False,
    aws_key_id=aws_access_key_id,
    aws_access_key=aws_secret_access_key,
    hardware_platform=HARDWARE_TARGET,
)
nlp.start(model_cache_folder="/app/model_cache", aws_access_key=aws_secret_access_key, aws_key_id=aws_access_key_id,
          hc_license=nlp_license, enterprise_nlp_secret=nlp_secret, visual_secret=visual_secret,
          visual=True if visual_secret else False, )
if model_ref:
    # Cache model, if not specified user must
    # mount a folder to /app/model_cache/ which has a folder named `served_model`
    pipe = nlp.load(model_ref)
    pipe.predict("init")
    pipe.save("/app/model/served_model")



import unittest
import pytest

from johnsnowlabs.frameworks.embedding_retrieval import JohnSnowLabsLangChainDocumentSplitter, \
    JohnSnowLabsLangChainMedicalDocumentSplitter


# https://colab.research.google.com/drive/1J7JpxIkYcOlm01otttJLD6iFLge43dZ9?usp=sharing

class LangChainTestCase(unittest.TestCase):
    def test_integration(self):
        from langchain.document_loaders import TextLoader
        from langchain.text_splitter import CharacterTextSplitter
        from langchain.vectorstores import FAISS
        from langchain.embeddings import OpenAIEmbeddings
        from langchain.agents.agent_toolkits import create_retriever_tool

        from johnsnowlabs.llm import embedding_retrieval

        p = "C:/Users/MONSTER/Desktop/socrates.txt"
        loader = TextLoader(p)
        documents = loader.load()
        text_splitter = CharacterTextSplitter(chunk_size=1000, chunk_overlap=0)
        texts = text_splitter.split_documents(documents)
        embeddings = embedding_retrieval.JohnSnowLabsLangChainEmbedder(
            "en.embed_sentence.bert_base_uncased"
        )  #  OpenAIEmbeddings()
        db = FAISS.from_documents(texts, embeddings)
        retriever = db.as_retriever()
        tool = create_retriever_tool(
            retriever,
            "search_state_of_union",
            "Searches and returns documents regarding the state-of-the-union.",
        )
        tools = [tool]

        from langchain.agents.agent_toolkits import create_conversational_retrieval_agent
        from langchain.chat_models import ChatOpenAI

        open_api_key = "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"
        llm = ChatOpenAI(temperature=0, openai_api_key=open_api_key)
        agent_executor = create_conversational_retrieval_agent(llm, tools, verbose=True)

        result = agent_executor(
            {"input": "what did the president say about going to east of Columbus?"}
        )
        result["output"]

    def test_doc_splitter(self):
        from langchain.docstore.document import Document
        documents = [
            Document(page_content="I like bananas \n\n and other things \n\n like icream \n\n and cats", metadata={"source": "local"}),
            Document(page_content="I like bananas \n\n and other things \n\n like icream \n\n and cats", metadata={"source": "local"}),
        ]
        splitter = JohnSnowLabsLangChainDocumentSplitter(chunk_size=20)
        result = splitter.split_documents(documents)
        print(result)

    def test_medical_doc_splitter(self):
        from langchain.docstore.document import Document
        documents = [
            Document(page_content="I like bananas \n\n and other things \n\n like icream \n\n and cats", metadata={"source": "local"}),
            Document(page_content="I like bananas \n\n and other things \n\n like icream \n\n and cats", metadata={"source": "local"}),
        ]
        splitter = JohnSnowLabsLangChainMedicalDocumentSplitter(split_mode="regex")
        result = splitter.split_documents(documents)
        print(result)



from johnsnowlabs.auto_install.databricks.install_utils import *
from johnsnowlabs.auto_install.databricks.install_utils import _get_cluster_id
from tests.databricks.db_test_utils import *
from tests.databricks.db_test_utils import get_or_create_test_cluster


@db_cloud_params
def test_list_workspace_infos(creds):
    # List infos about the workspace
    lic, host, token = creds
    db_client = get_db_client_for_token(host, token)
    list_db_runtime_versions(db_client)
    list_node_types(db_client)
    list_clusters(db_client)


# Parameterizing over both creds and node_type
@db_cloud_node_params
def test_get_db_cluster_infos(creds, node_type):
    # list infos specific to a cluster
    lic, host, token = creds
    db_client = get_db_client_for_token(host, token)

    cluster_id = get_or_create_test_cluster(creds, node_type, 0)
    list_cluster_lib_status(db_client, cluster_id)
    pprint(db_client.cluster.get_cluster(cluster_id))

    # Check cluster name+runtime detected
    assert (
        cluster_exist_with_name_and_runtime(
            db_client, "TEST_CLUSTER_0", settings.db_spark_version
        )
        == True
    )

    # check cluster id exists
    assert does_cluster_exist_with_id(db_client, cluster_id) == True
    # cluster_id = get_or_create_test_cluster(aws_creds, aws_cpu_node, 0)


@db_cloud_params
def test_delete_all_test_clusters(creds):
    lic, host, token = creds
    db_client = get_db_client_for_token(host, token)
    delete_all_test_clusters(db_client)
    for i in range(10):
        assert _get_cluster_id(db_client, f"TEST_CLUSTER_{i}") is None



import os.path

from tests.databricks.db_test_utils import *
from tests.databricks.db_test_utils import (
    assert_job_suc,
    run_cluster_test_suite,
    get_or_create_test_cluster,
)


def nlu_func():
    import nlu

    medical_text = """A 28-year-old female with a history of gestational
            diabetes presented with a one-week history of polyuria ,
             polydipsia , poor appetite , and vomiting ."""
    # TODO use licensed models
    # df = nlu.load("en.med_ner.diseases").predict(medical_text)
    df = nlu.load("tokenize").predict(medical_text)
    for c in df.columns:
        print(df[c])


@db_cloud_node_params
def test_run_integration_tests(creds, node_type):
    lic, host, token = creds
    cluster_id = get_or_create_test_cluster(creds, node_type, 0)
    run_cluster_test_suite(cluster_id, host, token)


@db_cloud_node_params
def test_submit_string_task_to_databricks(creds, node_type):
    cluster_id = get_or_create_test_cluster(creds, node_type, 0)
    lic, host, token = creds
    script = """
import nlu
print(nlu.load('sentiment').predict('That was easy!'))
        """
    assert_job_suc(
        nlp.run_in_databricks(
            script,
            databricks_cluster_id=cluster_id,
            databricks_host=host,
            databricks_token=token,
            run_name="Python Code String Example",
        )
    )


@db_cloud_node_params
def test_submit_local_py_file_task_to_databricks(creds, node_type):
    cluster_id = get_or_create_test_cluster(creds, node_type, 0)
    lic, host, token = creds
    # Test script
    import johnsnowlabs.auto_install.health_checks.nlp_test as nlp_test

    py_script_file_path = os.path.abspath(nlp_test.__file__)
    assert_job_suc(
        nlp.run_in_databricks(
            py_script_file_path,
            databricks_cluster_id=cluster_id,
            databricks_host=host,
            databricks_token=token,
            run_name="Script test",
        )
    )


@db_cloud_node_params
def test_submit_function_task_to_databricks(creds, node_type):
    cluster_id = get_or_create_test_cluster(creds, node_type, 0)
    lic, host, token = creds
    # basically reads the function code and submits it to databricks.
    # It also copies indentation and comments,watch out for that
    assert_job_suc(
        nlp.run_in_databricks(
            nlu_func,
            databricks_cluster_id=cluster_id,
            databricks_host=host,
            databricks_token=token,
            run_name="Function test",
        )
    )


@db_cloud_node_params
def test_submit_notebook_task_to_databricks(creds, node_type):
    cluster_id = get_or_create_test_cluster(creds, node_type, 0)
    lic, host, token = creds

    nb_path = "sample.ipynb"
    dst_path = "/Users/christian@johnsnowlabs.com/test.ipynb"
    # nb_path = "/home/ckl/Documents/tmp/databricks_endpoints_tutorial_PUBLIC.ipynb"
    assert_job_suc(
        nlp.run_in_databricks(
            nb_path,
            databricks_cluster_id=cluster_id,
            databricks_host=host,
            databricks_token=token,
            run_name="Notebook Test",
            dst_path=dst_path,
        )
    )


@db_cloud_node_params
def test_submit_notebook_parameterized_task_to_databricks(creds, node_type):
    # cluster_id = get_or_create_test_cluster(creds, node_type, 0)
    lic, host, token = creds
    cluster_id = "1005-134624-a6p9ji9u"

    nb_path = "parameterized_nb_example.ipynb"
    dst_path = "/Users/christian@johnsnowlabs.com/test.ipynb"
    assert_job_suc(
        nlp.run_in_databricks(
            nb_path,
            databricks_cluster_id=cluster_id,
            databricks_host=host,
            databricks_token=token,
            run_name="Notebook Test",
            dst_path=dst_path,
            parameters={"input_text": "How are you today", "model_name": "sentiment"},
        )
    )


@db_cloud_node_params
def test_submit_script_parameterized_task_to_databricks(creds, node_type):
    #    cluster_id = get_or_create_test_cluster(creds, node_type, 0)

    script = """
import sys
print(f"First argument: {sys.argv[1]}, Second argument: {sys.argv[2]}")
    """

    lic, host, token = creds
    cluster_id = "1005-134624-a6p9ji9u"

    arg1 = "My first arg"
    arg2 = "My second arg"
    assert_job_suc(
        nlp.run_in_databricks(
            script,
            databricks_cluster_id=cluster_id,
            databricks_host=host,
            databricks_token=token,
            run_name="Notebook Test",
            parameters=[arg1, arg2],
        )
    )



import json
import os
import sys
from enum import Enum
from typing import List

from fastapi import FastAPI, Request
from fastapi import UploadFile, File
from starlette.responses import JSONResponse

from johnsnowlabs import *

os.environ["PYSPARK_PYTHON"] = sys.executable
os.environ["PYSPARK_DRIVER_PYTHON"] = sys.executable
visual_enabled = 'VISUAL_SECRET' in os.environ
aws_access_key_id = os.environ.get("JOHNSNOWLABS_AWS_ACCESS_KEY_ID", None)
aws_secret_access_key = os.environ.get("JOHNSNOWLABS_AWS_SECRET_ACCESS_KEY", None)
hardware_target = os.environ.get("HARDWARE_TARGET", "cpu")
model_ref = os.environ.get("MODEL_TO_LOAD", None)

print(visual_enabled, aws_access_key_id, aws_secret_access_key, hardware_target, model_ref)

nlp_secret = os.environ.get("MEDICAL_SECRET", None)
nlp_license = os.environ.get("JOHNSNOWLABS_LICENSE", None)
visual_secret = os.environ.get("VISUAL_SECRET", None)

# # # # jars loaded from jsl-home
nlp.start(model_cache_folder="/app/model_cache", aws_access_key=aws_secret_access_key, aws_key_id=aws_access_key_id,
          hc_license=nlp_license, enterprise_nlp_secret=nlp_secret, visual_secret=visual_secret,
          visual=True if visual_secret else False, )

model = nlp.load(path="/app/model/served_model", verbose=True)
if visual_enabled:
    # TODO this needs to be set by NLU
    model.contains_ocr_components = True


# nlp.start()
# model = nlp.load('tokenize')

class OutputLevel(Enum):
    document = "document"
    sentence = "sentence"
    relation = "relation"
    chunk = "chunk"
    token = "token"


app = FastAPI()


@app.get("/predict")
async def predict(
        text: str = "Your text that you want to predict with the model goes here",
        output_level: OutputLevel = OutputLevel.document,
        positions: bool = False,
        metadata: bool = False,
        drop_irrelevant_cols: bool = False,
        get_embeddings: bool = False,  # true?
        keep_stranger_features: bool = True,
):
    return model.predict(
        text,
        output_level=output_level.value,
        positions=positions,
        metadata=metadata,
        drop_irrelevant_cols=drop_irrelevant_cols,
        get_embeddings=get_embeddings,
        keep_stranger_features=keep_stranger_features,
    ).to_json()


@app.post("/predict_batch")
async def predict_batch(
        text: List[str] = [
            "Your text that you want to predict with the model goes here",
            "More text can go here and will be interpreted as 2nd row",
        ],
        output_level: OutputLevel = OutputLevel.document,
        positions: bool = False,
        metadata: bool = False,
        drop_irrelevant_cols: bool = False,
        get_embeddings: bool = False,  # true?
        keep_stranger_features: bool = True,
):
    return model.predict(
        text,
        output_level=output_level.value,
        positions=positions,
        metadata=metadata,
        drop_irrelevant_cols=drop_irrelevant_cols,
        get_embeddings=get_embeddings,
        keep_stranger_features=keep_stranger_features,
    ).to_json()


@app.post("/predict_file")
async def upload_file(file: UploadFile = File(...)):
    contents = await file.read()
    file_path = f"/tmp/{file.filename}"
    with open(file_path, "wb") as f:
        f.write(contents)
    res = model.predict(file_path)
    if isinstance(res, List):
        return [df.to_json() for df in res]
    return res.to_json()


@app.post("/invoke")
async def invoke(request: Request):
    data = await request.json()
    text = [i[1] for i in data["data"]]
    if len(data['data']) > 0 and len(data['data']) > 1:
        output_level = data["data"][0][2]
    else:
        output_level = 'document'

    prediction = model.predict(
        text,
        output_level=output_level,
        positions=False,
        metadata=False,
        drop_irrelevant_cols=False,
        get_embeddings=False,
        keep_stranger_features=True,
    ).to_json(orient="table")
    json_content = json.loads(prediction)
    response = [[i["index"], i] for i in json_content["data"]]
    return JSONResponse(content={"data": response})


@app.get("/healthcheck")
def readiness_probe():
    return "I'm ready!"


@app.get("/ping")
def ping():
    return "I'm ready!"



from databricks_api import DatabricksAPI

from databricks_api import DatabricksAPI

from tests.databricks.db_test_utils import *
from tests.databricks.db_test_utils import (
    assert_job_suc,
)


@db_cloud_node_params
def test_create_cluster_with_custom_libs(creds, node_type):
    lic, host, token = creds
    extra_libs = ["farm-haystack==1.21.2", "langchain"]

    cluster_id = nlp.install_to_databricks(
        json_license_path=lic,
        databricks_host=host,
        databricks_token=token,
        driver_node_type_id=node_type,
        node_type_id=node_type,
        cluster_name="Test Custom Lib Cluster",
        clean_cluster=False,
        block_till_cluster_ready=True,
        extra_pip_installs=extra_libs,
    )
    script = """
import haystack
import langchain
print(haystack)
print(langchain)


    """
    # cluster_id = "1005-134624-a6p9ji9u"

    arg1 = "My first arg"
    arg2 = "My second arg"
    assert_job_suc(
        nlp.run_in_databricks(
            script,
            databricks_cluster_id=cluster_id,
            databricks_host=host,
            databricks_token=token,
            run_name="Notebook Test",
            parameters=[arg1, arg2],
        )
    )




@db_marketplace_cloud_node_params
def test_install_to_existing_cluster(creds, node_type):
    lic, host, token = creds
    print(lic, host, token)
    # Create a cluster with no installs
    db = DatabricksAPI(host=host, token=token)
    cluster_id = db.cluster.create_cluster(
        driver_node_type_id=node_type,
        node_type_id=node_type,
        spark_version=settings.db_spark_version,
        num_workers=1,
        cluster_name="Test Custom Lib Cluster6",
    )["cluster_id"]
    print(f"Created cluster {cluster_id}")
    wait_till_cluster_running(db, cluster_id)

    cluster_id = nlp.install_to_databricks(
        databricks_cluster_id=cluster_id,
        json_license_path=lic,
        databricks_host=host,
        databricks_token=token,
        driver_node_type_id=node_type,
        node_type_id=node_type,
        visual=True,
        clean_cluster=False,
        block_till_cluster_ready=True,
    )
    script = """
import nlu
nlu.load('en.embed_sentence.bert').predict('The quick brown fox jumps over the lazy dog')
    """
    # cluster_id = "1005-134624-a6p9ji9u"

    assert_job_suc(
        nlp.run_in_databricks(
            script,
            databricks_cluster_id=cluster_id,
            databricks_host=host,
            databricks_token=token,
            run_name="Notebook Test",
        )
    )



import unittest

from johnsnowlabs import JslSecrets
from johnsnowlabs.utils.venv_utils import *


class VenvTests(unittest.TestCase):
    test_dir = '/home/ckl/old_home/ckl/Documents/freelance/johnsnowlabs_lib/tmp/venv/tmp_test_venv'

    # venv_wrapper = VenvWrapper(self.venv_creation_dir, )
    def test_venv_wrapper(self):
        """
        Test all core features of the VenvWrapper, install, uninstall and check_if_contains_module
        """
        # Lib should not be found after having uninstalled it
        self.test_dir = '/home/ckl/old_home/ckl/Documents/freelance/johnsnowlabs_lib/tmp/venv/tmp_test_venv'
        VenvWrapper.create_venv(self.test_dir)
        VenvWrapper.uninstall_from_venv(self.test_dir, 'pyspark')
        self.assertTrue(VenvWrapper.is_lib_in_venv(self.test_dir, 'pyspark') is False)

        # After installing lib should be found
        self.assertTrue(VenvWrapper.install_to_venv(self.test_dir, 'pyspark'))
        self.assertTrue(VenvWrapper.is_lib_in_venv(self.test_dir, 'pyspark'))

        # Uninstall and check its missing
        VenvWrapper.uninstall_from_venv(self.test_dir, 'pyspark')
        self.assertTrue(VenvWrapper.is_lib_in_venv(self.test_dir, 'pyspark') is False)
        os.system(f'rm -r {self.test_dir} ')

    def test_install_jsl_suite_to_venv(self):
        f = '/home/ckl/old_home/ckl/Documents/freelance/johnsnowlabs_lib/tmp/licenses/ocr_40.json'
        secrets: JslSecrets = JslSecrets.build_or_try_find_secrets(secrets_file=f)


        print(secrets)
        # TODO
        pass

    # def test_jsl_suite_status(self):
    #     # TODO
    #     pass

    @classmethod
    def tearDownClass(cls):
        print("TEARING DOWN")
        os.system(f'rm -r {cls.test_dir} ')

    if __name__ == '__main__':
        unittest.main()



import re
import time
from pprint import pprint
from typing import List, Optional

from johnsnowlabs.auto_install import jsl_home
from johnsnowlabs.auto_install.softwares import Software
from johnsnowlabs.py_models.install_info import (
    InstallSuite,
    LocalPy4JLib,
    LocalPyLib,
    RootInfo,
)
from johnsnowlabs.py_models.lib_version import LibVersion
from johnsnowlabs.utils.env_utils import is_running_in_databricks
from .dbfs import *

# https://pypi.org/project/databricks-api/
from ...utils.enums import DatabricksClusterStates


def get_db_client_for_token(host, token) -> DatabricksAPI:
    # Get client via host or token
    return DatabricksAPI(host=host, token=token)


def get_db_client_for_password(host, email, password) -> DatabricksAPI:
    # Get client via user and password
    return DatabricksAPI(host=host, user=email, password=password)


def clean_cluster(
    databricks_host: str,
    databricks_token: str,
):
    dbfs_rm(
        get_db_client_for_token(databricks_host, databricks_token),
        settings.dbfs_home_dir,
        recursive=True,
    )


def install_license_to_cluster(install_suite: InstallSuite, db: DatabricksAPI, ):
    lic = {
        "SECRET": install_suite.secrets.HC_SECRET,
        "SPARK_OCR_SECRET": install_suite.secrets.OCR_SECRET,
        "SPARK_NLP_LICENSE": install_suite.secrets.HC_LICENSE,
        "SPARK_OCR_LICENSE": install_suite.secrets.OCR_LICENSE,
    }
    lic = {k: v for k, v in lic.items() if v is not None}

    put_file_on_dbfs(db, settings.dbfs_license_path, lic, overwrite=True)
    return settings.dbfs_license_path


def install_info_to_cluster(db: DatabricksAPI, ):
    info = RootInfo.get_from_jsl_home().dict()
    info["version"] = info["version"].as_str()
    put_file_on_dbfs(
        db,
        settings.dbfs_info_path,
        info,
        overwrite=True,
    )
    return settings.dbfs_info_path


def get_spark_conf(user_spark_conf):
    # Return merged user and default spark conf
    default_spark_conf = {
        "spark.kryoserializer.buffer.max": "2000M",
        "spark.serializer": "org.apache.spark.serializer.KryoSerializer",
        "spark.sql.optimizer.expression.nestedPruning.enabled": "false",
        "spark.sql.optimizer.nestedSchemaPruning.enabled": "false",
        "spark.sql.legacy.allowUntypedScalaUDF": "true",
        "spark.sql.repl.eagerEval.enabled": "true",
    }

    if not user_spark_conf:
        spark_conf = default_spark_conf
    else:
        user_spark_conf.update(default_spark_conf)
        spark_conf = user_spark_conf
    return spark_conf


def get_spark_env_vars(user_spark_env_vars, install_suite: InstallSuite, databricks_host,
                       databricks_token,
                       visual, medical_nlp, write_db_credentials,
                       ):
    # Return merged user and default spark env vars
    default_spark_env_vars = dict(
        SPARK_NLP_LICENSE_FILE=f"/dbfs/johnsnowlabs/license.json",
        AWS_ACCESS_KEY_ID=install_suite.secrets.AWS_ACCESS_KEY_ID,
        AWS_SECRET_ACCESS_KEY=install_suite.secrets.AWS_SECRET_ACCESS_KEY,
    )

    if install_suite.secrets.OCR_SECRET and visual:
        default_spark_env_vars["VISUAL_SECRET"] = install_suite.secrets.OCR_SECRET
    if install_suite.secrets.HC_SECRET and medical_nlp:
        default_spark_env_vars["HEALTHCARE_SECRET"] = install_suite.secrets.HC_SECRET

    if write_db_credentials:
        default_spark_env_vars["DATABRICKS_HOST"] = databricks_host
        default_spark_env_vars["DATABRICKS_TOKEN"] = databricks_token
    # Env vars may not be None, so we drop any that are None
    default_spark_env_vars = {
        k: v for k, v in default_spark_env_vars.items() if v is not None
    }

    if not user_spark_env_vars:
        spark_env_vars = default_spark_env_vars
    else:
        user_spark_env_vars.update(default_spark_env_vars)
        spark_env_vars = user_spark_env_vars
    return spark_env_vars


def create_cluster(
    databricks_host: str,
    databricks_token: str,
    medical_nlp,
    spark_nlp,
    visual,
    install_suite: InstallSuite = None,
    num_workers=1,
    cluster_name=settings.db_cluster_name,
    node_type_id=settings.db_node_type_id,
    driver_node_type_id=settings.db_driver_node_type,
    spark_env_vars=None,
    autotermination_minutes=60,
    spark_version=settings.db_spark_version,
    spark_conf=None,
    auto_scale=None,
    aws_attributes=None,
    ssh_public_keys=None,
    custom_tags=None,
    cluster_log_conf=None,
    enable_elastic_disk=None,
    cluster_source=None,
    instance_pool_id=None,
    headers=None,
    block_till_cluster_ready: bool = True,
    write_db_credentials: bool = True,
    extra_pip_installs: Optional[List[str]] = None,
) -> str:
    db = get_db_client_for_token(databricks_host, databricks_token)

    if not install_suite:
        install_suite = jsl_home.get_install_suite_from_jsl_home()

    install_license_to_cluster(install_suite, db)
    install_info_to_cluster(db)

    spark_env_vars = get_spark_env_vars(spark_env_vars, install_suite, databricks_host,
                                        databricks_token,
                                        visual, medical_nlp, write_db_credentials)
    spark_conf = get_spark_conf(spark_conf)

    cluster_id = db.cluster.create_cluster(
        num_workers=num_workers,
        autoscale=auto_scale,
        cluster_name=cluster_name,
        spark_version=spark_version,
        spark_conf=spark_conf,
        aws_attributes=aws_attributes,
        node_type_id=node_type_id,
        driver_node_type_id=driver_node_type_id,
        ssh_public_keys=ssh_public_keys,
        custom_tags=custom_tags,
        cluster_log_conf=cluster_log_conf,
        spark_env_vars=spark_env_vars,
        autotermination_minutes=autotermination_minutes,
        enable_elastic_disk=enable_elastic_disk,
        cluster_source=cluster_source,
        instance_pool_id=instance_pool_id,
        headers=headers,
    )["cluster_id"]
    print(f"👌 Created cluster with id={cluster_id} on host={db.client.url}")
    install_jsl_suite_to_cluster(
        db=db,
        cluster_id=cluster_id,
        install_suite=install_suite,
        medical_nlp=medical_nlp,
        spark_nlp=spark_nlp,
        visual=visual,
    )

    if extra_pip_installs:
        install_list_of_pypi_ref_to_cluster(db, cluster_id, extra_pip_installs)

    if block_till_cluster_ready:
        block_till_cluster_ready_state(db, cluster_id)

    return cluster_id


def list_db_runtime_versions(db: DatabricksAPI):
    versions = db.cluster.list_spark_versions()
    # pprint(versions)
    for version in versions["versions"]:
        print(version["key"])
        name = version["name"]
        # version_regex = r'[0-9].[0-9].[0-9]'

        spark_version = re.findall(r"Apache Spark [0-9].[0-9]", version["name"])
        if spark_version:
            spark_version = spark_version[0].lstrip("Apache Spark ")
        scala_version = re.findall(r"Scala [0-9].[0-9][0-9]", version["name"])[
            0
        ].lstrip("Scala ")
        has_gpu = len(re.findall("GPU", version["name"])) > 0
        if spark_version:
            spark_version = spark_version + ".x"
        version = LibVersion(spark_version).as_str()
        print(
            f"name={name}\n"
            f"version={version}\n"
            f"has_gpu={has_gpu}\n"
            f"scala_version={scala_version}\n"
            f"spark_version={spark_version}\n"
            f"{'=' * 25}"
        )


def list_cluster_lib_status(db: DatabricksAPI, cluster_id: str):
    lib_statuses = db.managed_library.cluster_status(cluster_id=cluster_id)
    # lib_statuses = db.managed_library.all_cluster_statuses()
    pprint(lib_statuses)
    return lib_statuses


def list_node_types(db: DatabricksAPI):
    node_types = db.cluster.list_node_types(headers=None)
    pprint(node_types)


def install_list_of_pypi_ref_to_cluster(db: DatabricksAPI, cluster_id, pip_installs):
    # expects list of pypy references, either 'numpy' or 'numpy==1.2.3'
    for p in pip_installs:
        if "==" in p:
            package, version = p.split("==")
        else:
            package, version = p, None
        install_py_lib_via_pip(db, cluster_id, package, version)


def install_to_existing_cluster(
        databricks_host: str,
        databricks_token: str,
        databricks_cluster_id: str,
        install_suite: InstallSuite,
        medical_nlp,
        spark_nlp,
        visual,
        write_db_credentials: bool = True,
        extra_pip_installs: Optional[List[str]] = None,

):
    db = get_db_client_for_token(databricks_host, databricks_token)
    install_license_to_cluster(install_suite, db)
    install_info_to_cluster(db)

    cluster_info = db.cluster.get_cluster(databricks_cluster_id)
    if 'spark_conf' in cluster_info['spec']:
        spark_conf = cluster_info['spec']['spark_conf']
    else:
        spark_conf = None
    if 'spark_env_vars' in cluster_info['spec']:
        spark_env_vars = cluster_info['spec']['spark_env_vars']
    else:
        spark_env_vars = None


    spark_env_vars = get_spark_env_vars(spark_env_vars, install_suite, databricks_host,
                                        databricks_token,
                                        visual, medical_nlp, write_db_credentials)
    spark_conf = get_spark_conf(spark_conf)

    db.cluster.edit_cluster(cluster_id=databricks_cluster_id,
                            spark_conf=spark_conf,
                            spark_env_vars=spark_env_vars,
                            num_workers=cluster_info['spec']['num_workers'],
                            spark_version=cluster_info['spec']['spark_version'],
                            node_type_id=cluster_info['spec']['node_type_id'],
                            driver_node_type_id=cluster_info['spec']['driver_node_type_id'],
                            cluster_name=cluster_info['spec']['cluster_name'],
                            autotermination_minutes=cluster_info['autotermination_minutes'],
                            )

    install_jsl_suite_to_cluster(
        db=db,
        install_suite=install_suite,
        cluster_id=databricks_cluster_id,
        medical_nlp=medical_nlp,
        spark_nlp=spark_nlp,
        visual=visual,
    )

    if extra_pip_installs:
        install_list_of_pypi_ref_to_cluster(
            db=db,
            cluster_id=databricks_cluster_id,
            pip_installs=extra_pip_installs,
        )


def install_jsl_suite_to_cluster(
    db: DatabricksAPI,
    cluster_id: str,
    install_suite: InstallSuite,
    medical_nlp: bool,
    spark_nlp: bool,
    visual: bool,
):
    py_deps = [
        {"package": Software.nlu.pypi_name, "version": settings.raw_version_nlu},
        {
            "package": Software.sparknlp_display.pypi_name,
            "version": settings.raw_version_nlp_display,
        },
        {
            "package": Software.jsl_lib.pypi_name_databricks,
            "version": settings.raw_version_jsl_lib,
        },
    ]
    uninstall_old_libraries(db, cluster_id, py_deps)
    if (
        install_suite.hc.get_py_path()
        and install_suite.hc.get_java_path()
        and medical_nlp
    ):
        install_py4j_lib_via_hdfs(db, cluster_id, install_suite.hc)
        print(
            f"Installed {Software.spark_hc.logo + Software.spark_hc.name} Spark NLP for Healthcare ✅"
        )
    if install_suite.ocr.get_py_path() and install_suite.ocr.get_java_path() and visual:
        install_py4j_lib_via_hdfs(db, cluster_id, install_suite.ocr)
        print(
            f"Installed {Software.spark_ocr.logo + Software.spark_ocr.name} Spark OCR ✅"
        )

    for dep in py_deps:
        install_py_lib_via_pip(db, cluster_id, dep["package"], dep["version"])
    # TODO remove when NLU is compatible with pandas >=2
    install_py_lib_via_pip(db, cluster_id, "pandas", "1.5.3")

    # Install Sparkr-NLP as last library, so we have the correct version
    if (
        install_suite.nlp.get_py_path()
        and install_suite.nlp.get_java_path()
        and spark_nlp
    ):
        install_py4j_lib_via_hdfs(db, cluster_id, install_suite.nlp)
        print(
            f"{Software.spark_nlp.logo + Software.spark_nlp.name} Installed Spark NLP! ✅"
        )

    # On databricks environment should restart current cluster to apply uninstalls and installations
    if is_running_in_databricks():
        try:
            restart_cluster(db, cluster_id)
        except:
            pass


def block_till_cluster_ready_state(db: DatabricksAPI, cluster_id: str):
    status = None
    while status != DatabricksClusterStates.RUNNING:
        # https://docs.databricks.com/dev-tools/api/latest/clusters.html#clusterclusterstate
        status = DatabricksClusterStates(db.cluster.get_cluster(cluster_id)["state"])
        print(f"Cluster-Id={cluster_id} not ready, status={status.value}")
        time.sleep(10)

    print(f"👌 Cluster-Id {cluster_id} is ready!")


def uninstall_old_libraries(
    db: DatabricksAPI,
    cluster_id: str,
    pypi_deps: List[dict],
):
    """
    Tell Cluster to uninstall old libraries
    # https://docs.databricks.com/dev-tools/api/latest/libraries.html
    https://docs.databricks.com/dev-tools/api/latest/libraries.html#install
    :param db:
    :param cluster_id:
    :param pypi_deps:
    :return:
    """
    uninstalls = []
    statuses = db.managed_library.cluster_status(cluster_id=cluster_id)
    file_names = [
        "spark_nlp",
        "spark_nlp_jsl",
        "spark_ocr",
    ]
    pypi_packages = [d["package"] for d in pypi_deps]
    for status in statuses.get("library_statuses", []):
        installation_types = ["jar", "whl", "pypi"]
        for typ in installation_types:
            path = status["library"].get(typ)
            if path:
                if typ == "pypi":
                    pkg = path.get("package")
                    if pkg in pypi_packages:
                        uninstalls.append({typ: {"package": pkg}})
                else:
                    for fil in file_names:
                        path = path.replace("/dbfs/", "dbfs:/")
                        if fil in path.replace("-", "_"):
                            uninstalls.append({typ: path})
    if uninstalls:
        db.managed_library.uninstall_libraries(
            cluster_id=cluster_id, libraries=uninstalls
        )


def install_py_lib_via_pip(
    db: DatabricksAPI, cluster_id: str, pypi_lib: str, version: Optional[str] = None
):
    """
    Tell Cluster to install via public pypi
    # https://docs.databricks.com/dev-tools/api/latest/libraries.html
    https://docs.databricks.com/dev-tools/api/latest/libraries.html#install
    :param db:
    :param cluster_id:
    :param pypi_lib:
    :param version:
    :return:
    """
    # By not defining repo, we will use default pip index
    pypi = dict(package=pypi_lib)
    if version:
        pypi["version"] = version
    payload = [dict(pypi=pypi)]
    db.managed_library.install_libraries(cluster_id=cluster_id, libraries=payload)
    if version:
        print(f"Installed {pypi_lib}=={version} ✅")
    else:
        print(f"Installed {pypi_lib} ✅")


def install_py4j_lib_via_hdfs(db: DatabricksAPI, cluster_id: str, lib: LocalPy4JLib):
    """
    1. Copy lib files to HDFS if not present
    2. Tell Cluster to install
    https://docs.databricks.com/dev-tools/api/latest/libraries.html#install
    :param db:
    :param cluster_id:
    :param lib:
    :return:
    """
    copy_p4j_lib_to_hdfs_if_not_present(db, lib)
    payload = [
        dict(jar=get_db_path(lib.java_lib)),
        dict(
            whl=get_db_path(lib.py_lib),
        ),
    ]
    db.managed_library.install_libraries(cluster_id=cluster_id, libraries=payload)


def copy_p4j_lib_to_hdfs_if_not_present(db: DatabricksAPI, lib: LocalPy4JLib):
    if not is_lib_on_dbfs_cluster(db, lib.java_lib):
        copy_lib_to_dbfs_cluster(db, lib.java_lib)
    if not is_lib_on_dbfs_cluster(db, lib.py_lib):
        copy_lib_to_dbfs_cluster(db, lib.py_lib)


def copy_py_lib_to_hdfs_if_not_present(db: DatabricksAPI, lib: LocalPyLib):
    if not is_lib_on_dbfs_cluster(db, lib.py_lib):
        copy_lib_to_dbfs_cluster(db, lib.py_lib)


def is_lib_on_dbfs_cluster(
    db: DatabricksAPI, local_info: Union[JvmInstallInfo, PyInstallInfo]
):
    dbfs_path = get_db_path(local_info)
    return dbfs_file_exists(db, dbfs_path)


def copy_lib_to_dbfs_cluster(
    db: DatabricksAPI, local_info: Union[JvmInstallInfo, PyInstallInfo]
):
    dbfs_path = get_db_path(local_info)
    if isinstance(local_info, JvmInstallInfo):
        local_path = f"{settings.java_dir}/{local_info.file_name}"
    elif isinstance(local_info, PyInstallInfo):
        local_path = f"{settings.py_dir}/{local_info.file_name}"
    else:
        raise Exception(f"Invalid lib install type to copy {type(local_info)}")
    return copy_from_local_to_hdfs(db, local_path=local_path, dbfs_path=dbfs_path)


def wait_till_cluster_running(db: DatabricksAPI, cluster_id: str, timeout=900):
    # https://docs.databricks.com/dev-tools/api/latest/clusters.html#clusterclusterstate
    import time

    start_time = time.time()

    while True:
        elapsed_time = time.time() - start_time
        if elapsed_time > timeout:
            print(
                "Timeout reached while waiting for the cluster to be running. Check cluster UI."
            )
            return False

        time.sleep(5)
        status = DatabricksClusterStates(db.cluster.get_cluster(cluster_id)["state"])

        if status == DatabricksClusterStates.RUNNING:
            return True
        elif status in [
            DatabricksClusterStates.PENDING,
            DatabricksClusterStates.RESIZING,
            DatabricksClusterStates.RESIZING,
        ]:
            continue
        elif status in [
            DatabricksClusterStates.TERMINATED,
            DatabricksClusterStates.TERMINATING,
            DatabricksClusterStates.ERROR,
            DatabricksClusterStates.UNKNOWN,
        ]:
            return False


def restart_cluster(db: DatabricksAPI, cluster_id: str):
    db.cluster.restart_cluster(cluster_id=cluster_id)


####
def list_clusters(db: DatabricksAPI):
    """Lists all clusters."""
    clusters = db.cluster.list_clusters()
    return clusters


def cluster_exist_with_name_and_runtime(
    db: DatabricksAPI, name: str, runtime: str
) -> bool:
    """Checks if a cluster with a specific name and runtime exists."""
    clusters = list_clusters(db)
    for cluster in clusters.get("clusters", []):
        if cluster["cluster_name"] == name and runtime in cluster["spark_version"]:
            return True
    return False


def does_cluster_exist_with_id(db: DatabricksAPI, cluster_id: str) -> bool:
    """Checks if a cluster with a specific ID exists."""
    try:
        cluster_info = db.cluster.get_cluster(cluster_id=cluster_id)
        if cluster_info and "cluster_id" in cluster_info:
            return True
    except Exception as e:
        # Handle specific exceptions based on the Databricks API error responses
        pass
    return False


def get_cluster_id(db: DatabricksAPI, cluster_name: str, runtime: str) -> str:
    """
    Retrieves the cluster ID based on the cluster name and runtime.

    If there are multiple candidates:
    - Returns any that's in the 'RUNNING' state if there is one.
    - If none are in the 'RUNNING' state, returns any.
    """
    clusters = list_clusters(db)
    running_clusters = []
    other_clusters = []

    for cluster in clusters.get("clusters", []):
        if (
            cluster["cluster_name"] == cluster_name
            and runtime in cluster["spark_version"]
        ):
            if cluster["state"] == "RUNNING":
                running_clusters.append(cluster["cluster_id"])
            else:
                other_clusters.append(cluster["cluster_id"])

    if running_clusters:
        return running_clusters[0]
    elif other_clusters:
        return other_clusters[0]
    else:
        raise Exception(
            f"No cluster found with name {cluster_name} and runtime {runtime}"
        )


def _get_cluster_id(db: DatabricksAPI, cluster_name: str) -> str:
    """
    Retrieves the cluster ID based on the cluster name.
    """
    clusters = list_clusters(db)
    running_clusters = []
    other_clusters = []

    for cluster in clusters.get("clusters", []):
        if cluster["cluster_name"] == cluster_name:
            # if cluster["state"] != "TERMINATED":
            other_clusters.append(cluster["cluster_id"])

    if running_clusters:
        return running_clusters[0]
    elif other_clusters:
        return other_clusters[0]
    else:
        return None  # Return None if no cluster found with the given name


#############



import unittest
from unittest.mock import patch

from johnsnowlabs import settings
from johnsnowlabs.py_models.jsl_secrets import JslSecrets
from johnsnowlabs.utils.my_jsl_api import LibSecretResponse


class JslSecretsTestCase(unittest.TestCase):
    def setUp(self):
        self.license = "license"
        self.valid_healthcare_secret =  settings.raw_version_secret_medical+ "-healthcare-secret"
        self.valid_ocr_secret = settings.raw_version_secret_ocr +  "-ocr-secret"

    @patch("johnsnowlabs.py_models.jsl_secrets.get_secrets")
    def test_enforce_versions_should_retrieve_matching_secrets(self, mock_get_secrets):
        mock_get_secrets.return_value = [
            LibSecretResponse(
                product="Healthcare NLP",
                version=settings.raw_version_secret_medical,
                secret=self.valid_healthcare_secret,
                isLatest=True,
            ),
            LibSecretResponse(
                product="Visual NLP",
                version=settings.raw_version_secret_ocr,
                secret=self.valid_ocr_secret,
                isLatest=True,
            ),
            LibSecretResponse(
                product="Healthcare NLP",
                version="x.x",
                secret="healthcare-invalid-secret",
                isLatest=False,
            ),
            LibSecretResponse(
                product="Visual NLP",
                version="x.x",
                secret="ocr-invalid-secret",
                isLatest=False,
            ),
        ]
        secrets = JslSecrets.enforce_versions(JslSecrets(OCR_LICENSE=self.license))
        self.assertEqual(secrets.OCR_LICENSE, self.license)
        self.assertEqual(secrets.HC_VERSION, settings.raw_version_secret_medical)
        self.assertEqual(secrets.OCR_VERSION, settings.raw_version_secret_ocr)
        self.assertEqual(secrets.HC_SECRET, self.valid_healthcare_secret)
        self.assertEqual(secrets.OCR_SECRET, self.valid_ocr_secret)


    @patch("johnsnowlabs.py_models.jsl_secrets.get_secrets")
    def test_should_return_existing_secret_if_no_matching_secret_found(self, mock_get_secrets):
        existing_secrets = JslSecrets(HC_LICENSE=self.license, OCR_SECRET="ocr-invalid-secret", HC_SECRET="healthcare-invalid-secret")
        mock_get_secrets.return_value = [
            LibSecretResponse(
                product="Healthcare NLP",
                version="x.x",
                secret="healthcare-invalid-secret",
                isLatest=False,
            ),
            LibSecretResponse(
                product="Visual NLP",
                version="x.x",
                secret="ocr-invalid-secret",
                isLatest=False,
            ),
        ]
        secrets = JslSecrets.enforce_versions(existing_secrets)
        self.assertEqual(secrets.HC_LICENSE, self.license)
        self.assertEqual(secrets.HC_SECRET, existing_secrets.HC_SECRET)
        self.assertEqual(secrets.OCR_SECRET,existing_secrets.OCR_SECRET)
 


import unittest
from johnsnowlabs.utils.enums import *
from johnsnowlabs.auto_install.lib_resolvers import *
import tests.utils.secrets as sct


class LibDependencyResolutionCase(unittest.TestCase):
    def test_list_ocr_lib_resolve(self):
        for spark_version in OcrLibResolver.compatible_spark_versions:
            for install_type in [PyInstallTypes.tar, PyInstallTypes.wheel, JvmHardwareTarget.cpu]:
                print(f'Testing Spark version ={spark_version.as_str()} and install type = {install_type}')
                dep = OcrLibResolver.get_dependency_url(secret=sct.OCR_SECRET,
                                                        spark_version_to_match=spark_version,
                                                        install_type=install_type)
                print(dep)
                self.assertTrue(dep.validate())

                dep = OcrLibResolver.get_dependency_url(secret=sct.OCR_SECRET,
                                                        spark_version_to_match=None,
                                                        install_type=install_type)
                print(dep)
                self.assertTrue(dep.validate())


    def test_list_hc_lib_resolve(self):
        for spark_version in HcLibResolver.compatible_spark_versions:
            for install_type in [PyInstallTypes.tar, PyInstallTypes.wheel, JvmHardwareTarget.cpu]:
                print(f'Testing Spark version ={spark_version.as_str()} and install type = {install_type}')
                dep = HcLibResolver.get_dependency_url(secret=sct.JSL_SECRET,
                                                       spark_version_to_match=spark_version,
                                                       install_type=install_type)
                dep = HcLibResolver.get_dependency_url(secret=sct.JSL_SECRET,
                                                       spark_version_to_match=None,
                                                       install_type=install_type)

                print(dep)
                self.assertTrue(dep.validate())
                dep = HcLibResolver.get_dependency_url(secret=sct.JSL_SECRET,
                                                       spark_version_to_match=None,
                                                       install_type=install_type)
                print(dep)
                self.assertTrue(dep.validate())


    def test_list_nlp_lib_resolve(self):
        for spark_version in NlpLibResolver.compatible_spark_versions:
            for install_type in [PyInstallTypes.tar, PyInstallTypes.wheel,
                                 JvmHardwareTarget.cpu, JvmHardwareTarget.gpu, JvmHardwareTarget.m1]:
                print(f'Testing Spark version ={spark_version.as_str()} and install type = {install_type}')
                dep = NlpLibResolver.get_dependency_url(
                                                       spark_version_to_match=spark_version,
                                                       install_type=install_type)
                print(dep)
                self.assertTrue(dep.validate())

                dep = NlpLibResolver.get_dependency_url(
                    spark_version_to_match=None,
                    install_type=install_type)
                print(dep)
                self.assertTrue(dep.validate())



if __name__ == '__main__':
    unittest.main()



import traceback

from johnsnowlabs.abstract_base.lib_resolver import try_import_lib
from johnsnowlabs.auto_install.softwares import Software
from johnsnowlabs.utils.env_utils import reverse_compatibility_import
from johnsnowlabs.utils.print_messages import log_outdated_lib, log_broken_lib


warning_logged = False

try:
    if try_import_lib("sparknlp_jsl") and try_import_lib("sparknlp"):
        from sparknlp_jsl.functions import *
        from sparknlp_jsl.training import *
        from sparknlp_jsl.annotator.windowed.windowed_sentence import (
            WindowedSentenceModel,
        )
        from sparknlp_jsl.annotator.ner.zero_shot_ner import ZeroShotNerModel
        from sparknlp_jsl.annotator import (
            GenericSVMClassifierApproach,
            GenericSVMClassifierModel,
            GenericLogRegClassifierApproach,
            GenericClassifierModel,
            AssertionLogRegModel,
            AssertionDLModel,
            DeIdentificationModel,
            DocumentLogRegClassifierModel,
            RelationExtractionModel,
            RelationExtractionDLModel,
            ChunkMergeModel,
            SentenceEntityResolverModel,
            ChunkMapperModel,
            BertSentenceChunkEmbeddings,
            ChunkKeyPhraseExtraction,
            NerDisambiguatorModel,
            EntityChunkEmbeddings,
            ZeroShotRelationExtractionModel,
            TFGraphBuilder,
            NerConverterInternal,
            ChunkConverter,
            ChunkFilterer,
            NerChunker,
            AssertionFilterer,
            AnnotationMerger,
            RENerChunksFilter,
            ChunkSentenceSplitter,
            DrugNormalizer,
            ChunkMapperFilterer,
            DateNormalizer,
            GenericClassifierModel,
            ReIdentification,
            Replacer,
            AssertionChunkConverter,
            AssertionLogRegApproach,
            AssertionDLApproach,
            DeIdentification,
            DocumentLogRegClassifierApproach,
            RelationExtractionApproach,
            ChunkMergeApproach,
            SentenceEntityResolverApproach,
            ChunkMapperApproach,
            NerDisambiguator,
            ContextualParserApproach,
            ContextualParserModel,
            GenericClassifierApproach,
            Router,
            NerQuestionGenerator,
            DocumentHashCoder,
            DocMapperModel,
            DocMapperApproach,
            NameChunkObfuscatorApproach,
            NameChunkObfuscator,
            DocumentMLClassifierApproach,
            DocumentMLClassifierModel,
            Resolution2Chunk,
            MedicalQuestionAnswering,
            NerTemplateRenderModel,
            AverageEmbeddings,
            Doc2ChunkInternal,
            Chunk2Token,
            ExtractiveSummarization,
            ChunkFiltererApproach,
            FewShotClassifierModel,
            FewShotClassifierApproach,
            InternalDocumentSplitter,
            Text2SQL,
            IOBTagger,
            DocumentFiltererByClassifier,
            Flattener,
        )
        from sparknlp_jsl.structured_deidentification import StructuredDeidentification
        from sparknlp_jsl.modelTracer import ModelTracer
        from sparknlp_jsl import training_log_parser, Deid
        from sparknlp_jsl.training_log_parser import ner_log_parser

        from sparknlp_jsl.base import FeaturesAssembler

        from sparknlp_jsl.annotator.resolution.resolver_merger import ResolverMerger

        from sparknlp_jsl.annotator import (
            MedicalDistilBertForSequenceClassification as DistilBertForSequenceClassification,
            MedicalBertForSequenceClassification as BertForSequenceClassification,
            MedicalBertForTokenClassifier as BertForTokenClassification,
            MedicalNerModel as NerModel,
            MedicalNerApproach as NerApproach,
            MedicalQuestionAnswering as QuestionAnswering,
            MedicalTextGenerator as TextGenerator,
            MedicalSummarizer as Summarizer,
            InternalDocumentSplitter as DocumentSplitter,
            NerConverterInternal as NerConverter,
            EntityRulerInternalApproach as EntityRulerApproach,
            EntityRulerInternalModel as EntityRulerModel,
            TextMatcherInternal as TextMatcher,
            TextMatcherInternalModel as TextMatcherModel,
            RegexMatcherInternal as RegexMatcher,
            RegexMatcherInternalModel as RegexMatcherModel,
        )
        from sparknlp_jsl.compatibility import Compatibility
        from sparknlp_jsl.pretrained import InternalResourceDownloader
        from sparknlp_jsl.eval import (
            NerDLMetrics,
            NerDLEvaluation,
            SymSpellEvaluation,
            POSEvaluation,
            NerCrfEvaluation,
            NorvigSpellEvaluation,
        )

except Exception as err:
    log_broken_lib(Software.spark_hc)
    print(f"Error Message : {err}")
    print(f"Error Trace: {traceback.format_exc()}")
    print("Performing reverse compatibility import for medical module")
    reverse_compatibility_import(__file__, globals())

if try_import_lib("sparknlp_jsl") and try_import_lib("sparknlp"):
    if not Software.spark_hc.check_installed_correct_version() and not warning_logged:
        warning_logged = True
        import sparknlp_jsl

        log_outdated_lib(Software.spark_hc, sparknlp_jsl.version())



import os
import unittest

from johnsnowlabs.utils.file_utils import file_to_str
from johnsnowlabs.utils.modelhub_markdown import test_markdown
from johnsnowlabs.utils.notebooks import test_ipynb
from johnsnowlabs import start
from johnsnowlabs import *
from johnsnowlabs import *


# lxml
class MarkdownTestTestCase(unittest.TestCase):
    def test_remote_markdown(self):
        nb_to_test = [
            'https://nlp.johnsnowlabs.com/2022/09/02/legner_roberta_zeroshot_en.html',
            'https://nlp.johnsnowlabs.com/2022/08/31/legpipe_deid_en.html',
            'https://nlp.johnsnowlabs.com/2022/10/02/legner_bert_large_courts_de.html',
        ]
        for n in nb_to_test:
            print(test_markdown(n))

    def test_local_markdown(self):
        url = 'https://raw.githubusercontent.com/JohnSnowLabs/spark-nlp/master/docs/_posts/josejuanmartinez/2022-08-30-legmulticlf_edgar_en.md'
        os.system(f'wget {url}')
        print(test_markdown('2022-08-30-legmulticlf_edgar_en.md'))

    def test_folder_of_markdown(self):
        urls = [
            'https://raw.githubusercontent.com/JohnSnowLabs/spark-nlp/master/docs/_posts/josejuanmartinez/2022-02-14-clinical_deidentification_es.md',
            'https://raw.githubusercontent.com/JohnSnowLabs/spark-nlp/master/docs/_posts/josejuanmartinez/2022-02-15-ner_deid_generic_augmented_es.md',
            'https://raw.githubusercontent.com/JohnSnowLabs/spark-nlp/master/docs/_posts/josejuanmartinez/2022-08-30-legmulticlf_edgar_en.md', ]
        for u in urls:
            os.system(f'wget {u}')
        test_markdown(os.getcwd())

if __name__ == '__main__':
    unittest.main()



import unittest

from johnsnowlabs.utils.file_utils import file_to_str
from johnsnowlabs.utils.notebooks import test_ipynb
from johnsnowlabs import start
from johnsnowlabs import *
from johnsnowlabs import *

# lxml
class WorkshopNotebookTestCase(unittest.TestCase):
    def test_remote_notebook(self):
        ocr_5 = 'https://raw.githubusercontent.com/JohnSnowLabs/spark-nlp-workshop/master/tutorials/Certification_Trainings/Healthcare/5.Spark_OCR.ipynb'
        ocr_51 = 'https://raw.githubusercontent.com/JohnSnowLabs/spark-nlp-workshop/master/tutorials/Certification_Trainings/Healthcare/5.1.Spark_OCR_Multi_Modals.ipynb'
        ocr_52 = 'https://raw.githubusercontent.com/JohnSnowLabs/spark-nlp-workshop/master/tutorials/Certification_Trainings/Healthcare/5.2.Spark_OCR_Deidentification.ipynb'
        nb_to_test = [
            ocr_5,
            ocr_51,
            ocr_52,
        ]
        for n in nb_to_test:
            test_ipynb(n)

    def test_download_workshop_repo_and_run(self):
        res = test_ipynb('WORKSHOP-FIN')
        res.to_csv('WORKSHOP-FIN.csv',index=False)

    def test_folder_of_notebooks(self):
        folder = '/home/ckl/old_home/ckl/Documents/freelance/johnsnowlabs_lib/tmp/nb_tests/latest_workshop/johnsnowlabs v1.0/Finance'
        res.to_csv('WORKSHOP-FIN.csv',index=False)

    def test_local_notebook(self):
        pass

    def test_parse(self):
        import ast
        f = '/home/ckl/.johnsnowlabs/tmp_tests/notebook_tests/5.2.Spark_OCR_Deidentification.ipynb.nb_converted.py'
        from ast import Expr
        a = ast.parse(file_to_str(f))

    bad_regex = [

    ]









    def test_nerModel(self):
        jsl.start()
        from sparknlp_jsl.finance import FinanceNerModel
        m = FinanceNerModel.pretrained("finner_deid", "en", 'finance/models')
        print("FOUND",m)







    def test_quick(self):
        s = """
        
spark = sparkocr.start(secret=SPARK_OCR_SECRET, 
                       nlp_version=PUBLIC_VERSION,
                       nlp_secret=SECRET,
                       nlp_internal=JSL_VERSION
                       )
        """

        r = r'sparkocr.start\(.*\)'
        import re
        print(re.findall(r, s, re.DOTALL))


if __name__ == '__main__':
    unittest.main()
